#include <mictcp.h>
#include <api/mictcp_core.h>

struct mic_tcp_sock g_sock;

/**
 * Permet de créer un socket entre l’application et MIC-TCP.
 * @return le descripteur du socket ou bien -1 en cas d'erreur.
 */
int mic_tcp_socket(start_mode sm) {
    int result;
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");
    result = initialize_components(sm); /* Appel obligatoire */
    set_loss_rate(0);

    g_sock.fd = result;
    g_sock.state = IDLE;

    return result;
}

/**
 * Permet d’attribuer une adresse à un socket.
 * @return 0 si succès, et -1 en cas d’échec.
 */
int mic_tcp_bind(int socket, mic_tcp_sock_addr addr) {
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");

    int result = 0; // TODO change in v2

    g_sock.addr = addr;

    return result;
}

/**
 * Met le socket en état d'acceptation de connexions.
 * @return 0 si succès, -1 si erreur.
 */
int mic_tcp_accept(int socket, mic_tcp_sock_addr *addr) {
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");

    int result = 0;  // TODO change in v2

    g_sock.state = ESTABLISHED;

    return result;
}

/**
 * Permet de réclamer l’établissement d’une connexion.
 * @return 0 si la connexion est établie, et -1 en cas d’échec
 */
int mic_tcp_connect(int socket, mic_tcp_sock_addr addr) {
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");

    int result = 0;  // TODO change in v2

    g_sock.addr = addr;
    g_sock.state = ESTABLISHED;

    return result;
}

/**
 * Permet de réclamer l’envoi d’une donnée applicative.
 * @return la taille des données envoyées, et -1 en cas d'erreur
 */
int mic_tcp_send(int socket, char *mesg, int mesg_size) {
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");

    struct mic_tcp_pdu pdu;
    pdu.payload.data = mesg;
    pdu.payload.size = mesg_size;

    pdu.header.source_port = 0;
    pdu.header.dest_port = g_sock.addr.port;
    pdu.header.seq_num = 0;
    pdu.header.ack_num = 0;
    pdu.header.syn = 0;
    pdu.header.ack = 0;
    pdu.header.fin = 0;

    return IP_send(pdu, g_sock.addr);
}

/**
 * Permet à l’application réceptrice de réclamer la récupération d’une donnée
 * stockée dans les buffers de réception du socket.
 * @return le nombre d’octets lu ou bien -1 en cas d’erreur.
 * NB : cette fonction fait appel à la fonction app_buffer_get()
 */
int mic_tcp_recv(int socket, char *mesg, int max_mesg_size) {
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");

    int result = -1;

    struct mic_tcp_payload payload;
    payload.data = malloc(max_mesg_size * sizeof(char));
    payload.size = max_mesg_size;
    int get_size = app_buffer_get(payload);
    memcpy(mesg, payload.data, get_size);
    free(payload.data);
    result = get_size;

    return result;
}

/**
 * Permet de réclamer la destruction d’un socket.
 * Engendre la fermeture de la connexion suivant le modèle de TCP.
 * @return 0 si tout se passe bien et -1 en cas d'erreur
 */
int mic_tcp_close(int socket) {
    printf("[MIC-TCP] Appel de la fonction :  ");
    printf(__FUNCTION__);
    printf("\n");

    g_sock.state = CLOSED;

    return -1;
}

/**
 * Traitement d’un PDU MIC-TCP reçu (mise à jour des numéros de séquence
 * et d'acquittement, etc.) puis insère les données utiles du PDU dans
 * le buffer de réception du socket. Cette fonction utilise la fonction
 * app_buffer_put().
 */
void process_received_PDU(mic_tcp_pdu pdu, mic_tcp_sock_addr addr) {
    printf("[MIC-TCP] Appel de la fonction: ");
    printf(__FUNCTION__);
    printf("\n");

    app_buffer_put(pdu.payload);
}
